@@ -4,12 +4,12 @@ from copy import deepcopy
 import numpy as np
 from scipy.ndimage.interpolation import map_coordinates, zoom
 
-from rotlib import rotx, roty, rotz
 from hdrio import imread
 
 from .tetrahedronSolidAngle import tetrahedronSolidAngle
 from .projections import *
 from .xmlhelper import EnvmapXMLParser
+from .rotations import rotx, roty, rotz
 
 
 SUPPORTED_FORMATS = [
@@ -413,6 +413,75 @@ class EnvironmentMap:
         :projection: perspective or orthographic.
         :mode: "normal": perform crop, "mask": show pixel mask in the envmap."""
 
+        coords = self._cameraCoordinates(vfov, rotation_matrix, ar,
+                                         resolution, projection, mode)
+
+        target = self.copy()
+        if target.format_ != "latlong":
+            target = target.convertTo("LatLong")
+
+        # Get image coordinates from world sphere coordinates
+        u, v = target.world2image(coords[0,:], coords[1,:], coords[2,:])
+        u, v = u.reshape(resolution[::-1]), v.reshape(resolution[::-1])
+
+        return target.interpolate(u, v).data
+    
+    def embed(self, vfov, rotation_matrix, image, projection="perspective"):
+        image = image.copy()
+        h, w, c = image.shape
+        resolution = [w, h]
+        ar = w/h
+        mode = "normal"
+
+        target = self.copy()
+        if target.format_ != "latlong":
+            target = target.convertTo("LatLong")
+
+        x, y, z, _ = target.worldCoordinates()
+        xyz = np.dstack((x, y, z))
+
+        coords = self._cameraCoordinates(vfov, rotation_matrix, ar,
+                                         resolution, projection, mode)
+
+        # Get image coordinates from world sphere coordinates
+        u, v = target.world2image(coords[0,:], coords[1,:], coords[2,:])
+        u, v = u.reshape(resolution[::-1]), v.reshape(resolution[::-1])
+
+        u = (np.round(u*(self.data.shape[1] - 1))).astype('int')
+        v = (np.round(v*(self.data.shape[0] - 1))).astype('int')
+        uv = np.dstack((u, v))
+
+        iu, iv = np.mgrid[0:image.shape[0], 0:image.shape[1]]
+
+        su, sv = self.imageCoordinates()
+ 
+        # raster
+        acc = np.zeros_like(self.data, dtype=np.float32)
+        du = np.diff(u, axis=1).max()
+        dv = np.diff(v, axis=0).max()
+
+        self.data[:] = np.nan
+        for dist_v in sorted(range(dv), key=abs):
+            i = dist_v - dv//2
+            for dist_u in sorted(range(du), key=abs):
+                j = dist_u - du//2
+                ve = ((v.copy() + i) % self.data.shape[0]) - i
+                ue = ((u.copy() + j) % self.data.shape[1]) - j
+                mask3 = np.isnan(self.data[ve+i,ue+j,:])
+                mask = np.logical_or.reduce((mask3[:,:,0], mask3[:,:,1], mask3[:,:,2]))
+                ve = ve[mask]
+                ue = ue[mask]
+                self.data[ve+i,ue+j,:] = image[mask]
+
+        mask3 = np.isnan(self.data)
+        mask = np.logical_or.reduce((mask3[:,:,0], mask3[:,:,1], mask3[:,:,2]))
+        self.data[mask] = self.backgroundColor[np.newaxis,:]
+
+        return self
+
+    def _cameraCoordinates(self, vfov, rotation_matrix, ar=4./3., resolution=(640, 480),
+                projection="perspective", mode="normal"):
+
         if mode not in ("normal", "mask"):
             raise Exception("Unknown mode: {}.".format(mode))
 
@@ -433,8 +502,8 @@ class EnvironmentMap:
             x = xy*np.sin(theta)
             y = xy*np.cos(theta)
 
-            hmask = (x>-mu) & (x<mu)
-            vmask = (y>-mv) & (y<mv)
+            hmask = (x > -mu) & (x < mu)
+            vmask = (y > -mv) & (y < mv)
             dmask = z < 0
             mask = hmask & vmask & dmask
 
@@ -466,15 +535,7 @@ class EnvironmentMap:
         # Perform rotation
         coords = rotation_matrix.T.dot(coords)
 
-        target = self.copy()
-        if target.format_ != "latlong":
-            target = target.convertTo("LatLong")
-
-        # Get image coordinates from world sphere coordinates
-        u, v = target.world2image(coords[0,:], coords[1,:], coords[2,:])
-        u, v = u.reshape(resolution[::-1]), v.reshape(resolution[::-1])
-
-        return target.interpolate(u, v).data
+        return coords
 
 
 def rotation_matrix(azimuth, elevation, roll=0):
